//
//	W-SIM Bluetooth(BGM203) I/F
//
//	MCU:		ATtiny2313
//
//		efuse:0xFF	自己プログラミング不許可
//		hfuse:0x9B	BOD 2.7V チップ消去から EEPROM保護
//		lfuse:0xEF	セラミック発振子 9.22MHz
//
//	Compiler:	AVR-GCC		3.4.6
//				AVR-LIBC	1.4.6
//				binutils	2.16.1
//
//	2008.2.9	1.0.2	ポートを変更可能にした
//	2008.2.8	1.0.1	デバイス名変更
//	2008.2.7	1.0.0	KAWAKAMI Yukio
//

#define		VERSION			"1.0.2"
#define		DEVICE_NAME		"W-SIM/BT"

#define		CPU_CLOCK		9216000		// 9.216MHzとして扱う

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdlib.h>

#include "WSIM-BT.h"

volatile uchar	Timer;
volatile uchar	FlashLED1;

//*******************************************************
//	タイマー１比較一致Ａ割り込み
//	100Hzで割り込みがかかる
//*******************************************************
ISR(TIMER1_COMPA_vect){
	static uchar count1;
	static uchar count2;
	static uchar led1;

	if (Timer)	--Timer;

	// LED1 点滅処理
	if (++count1 >= 12){
		count1 = 0;
		if (++count2 >= 8){	// 約1秒毎に点滅パターン更新
			led1 = FlashLED1;
			count2 = 0;
		}

		if (led1 & 0x80){
			LED1_ON;
		} else {
			LED1_OFF;
		}

		led1 <<= 1;
	}
}


//*******************************************************
//	メインルーチン
//*******************************************************
int main(void){
	uchar	s;
	uchar	port;

	cli();

	PORTB = (1<<PB7)|(1<<PB6)|(1<<PB5)|(1<<G244)|(1<<STAT);
	DDRB = (1<<LED3)|(1<<LED2)|(1<<LED1)|(1<<G244);
	
	PORTD = (1<<DISP3)|(1<<DISP2)|(1<<DISP1)|(1<<LBO)|(1<<TXD)|(1<<RXD);
	DDRD = (1<<RESET);

	MCUCR = (1<<SE);	// スリープ許可・アイドルモード
	
	// タイマ１
	TCCR1A = 0;						// CTC動作
	TCCR1B = (1<<WGM12)|(1<<CS12)|(1<<CS10);	// プリスケーラ 1/1024
	OCR1AH = (CPU_CLOCK/1024/100 - 1) >> 8;
	OCR1AL = (uchar)(CPU_CLOCK/1024/100 - 1);	// 100Hz
	TCNT1H = 0;
	TCNT1L = 0;
	TIMSK = (1<<OCIE1A);			// 比較A割り込み許可

	ACSR = (1<<ACD);			// アナログコンパレータ禁止
	
	sei();

	FlashLED1 = LED_BUSY;		// BUSY点滅

	// １秒時間待ち(電源が安定するまで待つ)
	wait_1s();

	while (!(PIND & (1<<LBO))){		// Low Battery
		FlashLED1 = LED_LB;
		wait_1s();
	}
	
	// USART設定
	UCSRA = (1<<U2X);				// ボーレート倍速
	UCSRB = (1<<RXEN)|(1<<TXEN);	// 送受信許可
	UCSRC = (1<<UCSZ1)|(1<<UCSZ0);	// 8bit, Non, Stop 1

	// Bluetoothモジュールリセット解除
	BLUETOOTH_ON;
	
	// １秒時間待ち(Bluetoothモジュールが起動するまで待つ)
	wait_1s();

	// 9600bpsで反応確認
	UBRRH = (CPU_CLOCK/8/9600 - 1) >> 8;
	UBRRL = (uchar)(CPU_CLOCK/8/9600 - 1);	// 9600bps
	if (send_command_P(PSTR("AT\r"))){
		if (!send_command_P(PSTR("+++"))){
			// Automatic connection mode OFF(フラッシュ保存)
			send_command_P(PSTR("AT+BTAUT=0\r"));
			send_command_P(PSTR("AT+BTFLS\r"));	// フラッシュ書き込み
			wait_1s();
		} else {
			SLEEP;
			SLEEP;
			// 230400bpsで反応確認
			UBRRH = (CPU_CLOCK/8/230400 - 1) >> 8;
			UBRRL = (uchar)(CPU_CLOCK/8/230400 - 1);		// 230400bps
			if (!send_command_P(PSTR("AT\r"))){
				// Automatic connection mode OFF(フラッシュ保存)
				send_command_P(PSTR("AT+BTAUT=0\r"));
				send_command_P(PSTR("AT+BTFLS\r"));	// フラッシュ書き込み
				wait_1s();
			} else {
				if (send_command_P(PSTR("+++"))){
					FlashLED1 = LED_ERROR;	// LED1 ERROR点滅
					LED2_ON;				// LED2 点灯
					BLUETOOTH_OFF;			// Bluetooth停止
					for (;;){ SLEEP; }		// 停止
				}
			}
		}
	}

	// エスケープシーケンス OFF
	// DTR/DSR/RI パススルー ON
	// ACTIVE出力 ON
	// DTEモード
	send_command_P(PSTR("AT+BTCFG=39\r"));	// DTE
	SLEEP;
	SLEEP;
	
	// ボーレート 230400bpsに変更
	send_command_P(PSTR("AT+BTURT=230400,8,0,1,1\r"));	// RTS/CTSフローあり
	SLEEP;
	SLEEP;

	// 230400bpsに変更
	UBRRH = (CPU_CLOCK/8/230400 - 1) >> 8;
	UBRRL = (uchar)(CPU_CLOCK/8/230400 - 1);		// 230400bps

	// Bluetooth デバイスクラス Telephony + Phone
	send_command_P(PSTR("AT+BTCOD=\\x400200\r"));

	// デバイス名	W-SIM/BT
	send_command_P(PSTR("AT+BTLNM=\"" DEVICE_NAME "\"\r"));

	// ローパワーモード ON
	send_command_P(PSTR("AT+BTPWR=1,1000,320,480,16,8\r"));

	// セキュリティ設定
	s = eeprom_read_byte(EEPROM_SECURITY_LEVEL);
	if ((s == 1)||(s == 2)){
		uint8_t *p;
		uchar	i;
		uchar	c;

		p = EEPROM_PIN_CODE;
		c = eeprom_read_byte(p);
		if ((c != 0)&&(c != 0xFF)){
			put_str_P(PSTR("AT+BTSRC="));
			put_char(s + '0');
			send_command_P(PSTR("\r"));

			put_str_P(PSTR("AT+BTPIN=\""));
			for (i = 0; i < 16; i++){	// 最大16bytes(128bit)
				c = eeprom_read_byte(p++);
				if ((c == 0)||(c == 0xFF))	break;
				put_char(c);
			}
			send_command_P(PSTR("\"\r"));
		}
	} else {
		// セキュリティ OFF
		send_command_P(PSTR("AT+BTSEC=0\r"));
	}
	
	// Automatic connection mode(フラッシュに保存しない)
	send_command_P(PSTR("AT+BTAUT=1\r"));

	// SPPサーバ動作開始
	port = eeprom_read_byte(EEPROM_SERVICE_PORT);	// Port番号取得
	if ((port < 1)||(port > 30)){
		port = 1;
	}
	put_str_P(PSTR("AT+BTSRV="));
	if (port >= 10){
		put_char((port / 10) + '0');
	}
	put_char((port % 10) + '0');
	if (send_command_P(PSTR(",\"" DEVICE_NAME "\"\r"))){
		FlashLED1 = LED_ERROR;	// LED1 ERROR点滅
		LED3_ON;				// LED3 点灯
		BLUETOOTH_OFF;			// Bluetooth停止
		for (;;){ SLEEP; }		// 停止
	}

	UCSRB = 0;						// USART停止
	PORTD &= ~((1<<TXD)|(1<<RXD));	// 内部プルアップ停止

	// W-SIMゲートON
	PORTB &= ~(1<<G244);
	FlashLED1 = LED_ON;			// LED1 常時点灯

	// あとは LED点滅処理のみ
	for (;;){

		SLEEP;	// 10ms寝る

		if (!(PIND & (1<<LBO))){	// Low Battery
			FlashLED1 = LED_LB;		// LED1 点滅
		} else {
			FlashLED1 = LED_ON;
		}
		
		if (!(PINB & (1<<STAT))){	// Bluetooth接続状態
			LED2_ON;
		} else {
			LED2_OFF;
		}

		if (!(PIND & (1<<DISP3))){	// W-SIM通信状態
			LED3_ON;
		} else {
			LED3_OFF;
		}
	}
}


//*******************************************************
//	PSTR コマンドを送信して反応(OK\r\n)を待つ
//*******************************************************
char  send_command_P(PGM_VOID_P com){

	put_str_P(com);
	UCSRA &= ~((1<<RXC)|(1<<FE)|(1<<DOR)|(1<<UPE));		// 受信関係フラグクリア

	return wait_ok(100);	// １秒
}


//*******************************************************
//	RAM コマンドを送信して反応(OK\r\n)を待つ
//*******************************************************
char  send_command(uchar *com){

	put_str(com);
	UCSRA &= ~((1<<RXC)|(1<<FE)|(1<<DOR)|(1<<UPE));		// 受信関係フラグクリア

	return wait_ok(100);	// １秒
}


//*******************************************************
//	OK\r\n が来るまで待つ
//	時間切れ判定
//*******************************************************
char wait_ok(uchar time_out){
	ulong	res;

	res = 0;
	Timer = time_out;
	while(res != OK_CRLF){
		while(!(UCSRA & (1<<RXC))){
			if (!Timer){
				return -1;		// 時間切れ
			}
		}
		res <<= 8;
		res |= UDR;
	}
	return 0;
}


//*******************************************************
//	シリアルに PSTR文字列出力
//*******************************************************
void put_str_P(PGM_VOID_P str){
	uchar	c;

	while((c = pgm_read_byte(str))){
		str++;
		put_char(c);
	}
}


//*******************************************************
//	シリアルに 文字列出力
//*******************************************************
void put_str(uchar *str){
	uchar	c;

	while((c = *str)){
		str++;
		put_char(c);
	}
}


//*******************************************************
//	 シリアルに一文字出力
//*******************************************************
void put_char(uchar c){
	
	while(!(UCSRA & (1<<UDRE)));
	UDR = c;
}


//*******************************************************
//	1秒時間待ち
//*******************************************************
void wait_1s(void){

	Timer = 100;
	while(Timer){
		SLEEP;
	}
}
